@@ -60,27 +60,29 @@ def lsq_gna_parallel(
     else:
         jac_fun = lambda p: jac_function(p, *args)
 
+    assert len(p.shape) == 2, 'parameter tensor is supposed to have 2 dims, but has %s' % str(len(p.shape))
+
     f = fun(p)
-    assert len(f.shape) == 3, 'residual tensor is supposed to have 3 dims'
+    assert len(f.shape) == 3, 'residual tensor is supposed to have 3 dims, but has %s' % str(len(f.shape))
 
     j = jac_fun(p)
-    assert len(j.shape) == 4, 'jacobian tensor is supposed to have 4 dims'
+    assert len(j.shape) == 4, 'jacobian tensor is supposed to have 4 dims, but has %s' % str(len(j.shape))
 
     # use weights of ones as default
     wvec = torch.ones(f.shape[1], dtype=p.dtype, device=p.device, requires_grad=False) if wvec is None else wvec
-    assert len(wvec) == f.shape[1], 'weights vector length is supposed to match number of costs'
+    assert len(wvec) == f.shape[1], 'weights vector length of %s is supposed to match cost number which is %s' % (str(len(wvec)), str(f.shape[1]))
 
     p_list = []
     f = torch.zeros_like(f)
     while len(p_list) < max_iter:
         f_prev = f.clone()
         p, f, g, h = gauss_newton_step(p, fun, jac_fun, wvec, l)
-        p_list.append(p.detach())
+        p_list.append(p.clone())
 
         # stop conditions
         gcon = torch.max(abs(g)) < gtol
         pcon = (h**2).sum()**.5 < ptol*(ptol + (p**2).sum()**.5)
-        fcon = ((f_prev-f)**2).sum() < ((ftol*f)**2).sum()
+        fcon = ((f_prev-f)**2).sum() < ((ftol*f)**2).sum() if f_prev.shape == f.shape else False
         if gcon or pcon or fcon:
             break
 
